@@ -1,30 +1,41 @@
-
 import builtins
 import copy
-import functools
 import inspect
 import sys
 import types
 
 from collections import defaultdict, deque
-from typing import TypeVar, Any, Optional, Callable, Type, Iterable, Mapping, Union, Tuple, List
+from typing import *
+from typing_extensions import ParamSpec
+
+from .errors import *
 
 __all__ = [
-    'common_ancestor', 'create_class', 'resolve_bases',
-    'static_vars', 'static_copy', 'static_hasattr', 'static_mro',
-    'resolve_identifier', 'is_sub_qualname',
-    'is_abstract',
-    'iter_wrapped', 'unwrap', 'extract_functions',
-    'rename', 'wraps',
-    'compile_function',
-    'camel_to_snake', 'snake_to_camel',
+    "common_ancestor",
+    "create_class",
+    "resolve_bases",
+    "static_vars",
+    "static_copy",
+    "static_hasattr",
+    "static_mro",
+    "resolve_identifier",
+    "is_sub_qualname",
+    "is_abstract",
+    "iter_wrapped",
+    "unwrap",
+    "extract_functions",
+    "rename",
+    "compile_function",
+    "camel_to_snake",
+    "snake_to_camel",
 ]
 
-T = TypeVar('T')
+P = ParamSpec("P")
+T = TypeVar("T")
 
 
-TYPE_GET_DICT = type.__dict__['__dict__'].__get__
-TYPE_GET_MRO = type.__dict__['__mro__'].__get__
+TYPE_GET_DICT = type.__dict__["__dict__"].__get__
+TYPE_GET_MRO = type.__dict__["__mro__"].__get__
 
 
 def create_class(
@@ -32,7 +43,7 @@ def create_class(
     bases: Iterable = (),
     attrs: dict = {},
     metaclass: Optional[Callable[..., Type[T]]] = None,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Type[T]:
     """
     Creates a new class. This is similar to :func:`types.new_class`, except it
@@ -47,7 +58,7 @@ def create_class(
     :return: A new class
     """
     if metaclass is not None:
-        kwargs.setdefault('metaclass', metaclass)
+        kwargs.setdefault("metaclass", metaclass)
 
     resolved_bases = resolve_bases(bases)
     meta, ns, kwds = types.prepare_class(name, resolved_bases, kwargs)
@@ -56,12 +67,12 @@ def create_class(
 
     # Note: In types.new_class this is "is not" rather than "!="
     if resolved_bases != bases:
-        ns['__orig_bases__'] = bases
+        ns["__orig_bases__"] = bases
 
     return meta(name, resolved_bases, ns, **kwds)
 
 
-def resolve_bases(bases: Iterable) -> tuple:
+def resolve_bases(bases: Iterable) -> Tuple[type]:
     """
     Clone/backport of :func:`types.resolve_bases`.
 
@@ -87,7 +98,17 @@ def resolve_bases(bases: Iterable) -> tuple:
     return tuple(result)
 
 
-def static_vars(obj: Any) -> Mapping:
+@overload
+def static_vars(obj: type) -> Mapping[str, object]:
+    ...
+
+
+@overload
+def static_vars(obj: object) -> Dict[str, object]:
+    ...
+
+
+def static_vars(obj: object) -> Mapping[str, object]:
     """
     Like :func:`vars`, but bypasses overridden ``__getattribute__`` methods and
     (for the most part) ``__dict__`` descriptors.
@@ -100,14 +121,14 @@ def static_vars(obj: Any) -> Mapping:
             ...
             >>> static_vars(Demo())
             'oops'
-        
+
         This is because I can't find a reliable way to access the real
         ``__dict__`` from within python. As far as I can tell, it's only
         possible through the C API.
 
     :param obj: An object
     :return: The object's ``__dict__``
-    :raises TypeError: If the object has no ``__dict__``
+    :raises ObjectHasNoDict: If the object has no ``__dict__``
     """
     # We'll start by invoking the __dict__ slot from the `type` class. That'll
     # fail with a TypeError if the object is not a class.
@@ -115,18 +136,18 @@ def static_vars(obj: Any) -> Mapping:
         return TYPE_GET_DICT(obj)
     except TypeError:
         pass
-    
+
     obj_type = type(obj)
-    
+
     # If it's not a class, we'll just grab the last __dict__ attribute from the
     # MRO, which will *hopefully* be a slot
     for cls in reversed(static_mro(obj_type)):
         cls_dict = static_vars(cls)
 
-        if '__dict__' not in cls_dict:
+        if "__dict__" not in cls_dict:
             continue
 
-        dict_slot = cls_dict['__dict__']
+        dict_slot = cls_dict["__dict__"]
 
         try:
             descriptor_get = dict_slot.__get__
@@ -135,7 +156,7 @@ def static_vars(obj: Any) -> Mapping:
         else:
             return descriptor_get(obj, obj_type)
 
-    raise TypeError(f"{obj!r} object has no __dict__")
+    raise ObjectHasNoDict("obj", obj)
 
 
 def static_copy(obj: T) -> T:
@@ -165,20 +186,20 @@ def static_copy(obj: T) -> T:
 
     # We'll check the __module__ attribute for speed and then also
     # make sure the class isn't lying about its module
-    if cls.__module__ == 'builtins' and cls in vars(builtins).values():
+    if cls.__module__ == "builtins" and cls in vars(builtins).values():
         return copy.copy(obj)
 
     new_obj = object.__new__(cls)
 
     try:
         old_dict = static_vars(obj)
-    except TypeError:
+    except ObjectHasNoDict:
         pass
     else:
         static_vars(new_obj).update(old_dict)
 
     for slot_name, slot in iter_slots(cls):
-        if slot_name in {'__dict__', '__weakref__'}:
+        if slot_name in {"__dict__", "__weakref__"}:
             continue
 
         try:
@@ -204,16 +225,16 @@ def static_hasattr(obj: Any, attr_name: str) -> bool:
     """
     try:
         obj_dict = static_vars(obj)
-    except TypeError:
+    except ObjectHasNoDict:
         pass
     else:
         if attr_name in obj_dict:
             return True
-    
+
     for cls in static_mro(type(obj)):
         if attr_name in vars(cls):
             return True
-        
+
         # Note: There's no need to handle give __slots__ any
         # special treatment, since they create descriptors in
         # the class namespace anyway.
@@ -221,7 +242,7 @@ def static_hasattr(obj: Any, attr_name: str) -> bool:
     return False
 
 
-def static_mro(cls: type) -> Tuple[type]:
+def static_mro(cls: type) -> Tuple[type, ...]:
     """
     Given a class as input, returns the class's MRO without invoking any
     overridden ``__getattribute__`` methods or ``__mro__`` descriptors.
@@ -268,9 +289,9 @@ def common_ancestor(classes: Iterable[type]) -> type:
 
 
 def iter_wrapped(
-        function: Union[Callable, staticmethod, classmethod],
-        stop: Optional[Callable[[Callable], bool]] = None,
-    ):
+    function: Union[Callable, staticmethod, classmethod],
+    stop: Optional[Callable[[Callable], bool]] = None,
+) -> Iterator[Callable]:
     """
     Given a function as input, yields the function and all the functions it
     wraps.
@@ -282,6 +303,9 @@ def iter_wrapped(
     and returns a truth value. As soon as ``stop(function)`` returns a truthy
     result, the iteration stops. (``function`` will not be yielded.)
 
+    Note that unlike :func:`unwrap`, this function doesn't raise an exception if
+    the input is a bound method.
+
     .. versionadded:: 1.4
 
     :param function: The function to unwrap
@@ -306,13 +330,16 @@ def iter_wrapped(
 
 
 def unwrap(
-        function: Union[Callable, staticmethod, classmethod],
-        stop: Optional[Callable[[Callable], bool]] = None,
-    ):
+    function: Union[Callable, staticmethod, classmethod],
+    stop: Optional[Callable[[Callable], bool]] = None,
+) -> Callable:
     r"""
     Like :func:`inspect.unwrap`, but always unwraps :class:`staticmethod`\ s and
     :class:`classmethod`\ s. (:func:`inspect.unwrap` only does this since 3.10)
 
+    Also unlike :func:`inspect.unwrap`, this function raises
+    :exc:`CannotUnwrapBoundMethod` if the input is a bound method.
+
     If ``stop`` is given, it must be a function that takes a function as input
     and returns a truth value. As soon as ``stop(function.__wrapped__)`` returns
     a truthy result, the unwrapping stops.
@@ -322,9 +349,12 @@ def unwrap(
     :param function: The function to unwrap
     :param stop: A predicate function indicating when to stop iterating
     """
+    if inspect.ismethod(function):
+        raise CannotUnwrapBoundMethod(function)
+
     for function in iter_wrapped(function, stop):
         pass
-    
+
     return function
 
 
@@ -343,15 +373,11 @@ def extract_functions(obj: Union[FUNCTION_CONTAINER_TYPES]) -> List[Callable]:
     """
     if isinstance(obj, (staticmethod, classmethod)):
         return [obj.__func__]
-    
+
     if isinstance(obj, property):
-        return [
-            func
-            for func in (obj.fget, obj.fset, obj.fdel)
-            if func is not None
-        ]
-    
-    raise TypeError(f'Cannot extract functions from {type(obj)!r} object')
+        return [func for func in (obj.fget, obj.fset, obj.fdel) if func is not None]
+
+    raise InvalidArgumentType("obj", obj, Union[FUNCTION_CONTAINER_TYPES])
 
 
 def is_abstract(obj: Any) -> bool:
@@ -371,11 +397,8 @@ def is_abstract(obj: Any) -> bool:
     :param obj: The object to inspect
     """
     if isinstance(obj, FUNCTION_CONTAINER_TYPES):
-        return any(
-            is_abstract(func)
-            for func in extract_functions(obj)
-        )
-    
+        return any(is_abstract(func) for func in extract_functions(obj))
+
     if isinstance(obj, type):
         seen = set()
 
@@ -387,10 +410,10 @@ def is_abstract(obj: Any) -> bool:
 
                 if is_abstract(value) and not isinstance(value, type):
                     return True
-        
+
         return False
 
-    return bool(getattr(obj, '__isabstractmethod__', False))
+    return bool(getattr(obj, "__isabstractmethod__", False))
 
 
 def rename(obj: Any, name: str) -> None:
@@ -405,14 +428,14 @@ def rename(obj: Any, name: str) -> None:
     old_qualname = obj.__qualname__
 
     obj.__name__ = name
-    obj.__qualname__ = old_qualname[:old_qualname.rfind('.')+1] + name
+    obj.__qualname__ = old_qualname[: old_qualname.rfind(".") + 1] + name
 
     # If it's a class, update the qualnames of its methods
     if not isinstance(obj, type):
         return
-    
-    prefix = old_qualname + '.'
-    
+
+    prefix = old_qualname + "."
+
     for attr in static_vars(obj).values():
         # staticmethods and classmethods have their own  __qualname__ that we
         # must update
@@ -423,60 +446,18 @@ def rename(obj: Any, name: str) -> None:
                 functions.append(attr)
         elif isinstance(attr, FUNCTION_CONTAINER_TYPES):
             functions = extract_functions(attr)
-        elif hasattr(attr, '__qualname__'):
+        elif hasattr(attr, "__qualname__"):
             functions = [attr]
         else:
             continue
-        
+
         for func in functions:
             if not func.__qualname__.startswith(prefix):
                 continue
 
-            func.__qualname__ = obj.__qualname__ + func.__qualname__[len(old_qualname):]
-
-
-def wraps(
-    wrapped_func: Callable,
-    name: Optional[str] = None,
-    signature: Optional[inspect.Signature] = None,
-    remove_parameters: Optional[Iterable[Union[str, int]]] = None,
-) -> Callable[[Callable], Callable]:
-    """
-    Similar to :func:`functools.wraps`, but allows you to modify the function's
-    metadata.
-
-    .. versionadded:: 1.4
-
-    :param wrapped_func: The wrapped function
-    :param name: A new name for the wrapper function
-    :param signature: A new signature for the wrapper function
-    :param remove_parameters: Parameter names or indices to remove from the
-        wrapper function's signature
-    """
-    from .signature import Signature
-    
-    def wrapper(wrapper_func):
-        functools.update_wrapper(wrapper_func, wrapped_func)
-
-        if name is not None:
-            rename(wrapper_func, name)
-        
-        if signature is not None or remove_parameters is not None:
-            if signature is None:
-                sig = Signature.from_callable(wrapped_func)
-            elif isinstance(signature, inspect.Signature):
-                sig = Signature.from_signature(signature)
-            else:
-                sig = Signature.from_callable(signature)
-            
-            if remove_parameters:
-                sig = sig.without_parameters(*remove_parameters)
-            
-            wrapper_func.__signature__ = sig
-
-        return wrapper_func
-
-    return wrapper
+            func.__qualname__ = (
+                obj.__qualname__ + func.__qualname__[len(old_qualname) :]
+            )
 
 
 def resolve_identifier(identifier: str) -> object:
@@ -487,7 +468,7 @@ def resolve_identifier(identifier: str) -> object:
 
         >>> resolve_identifier('introspection.Parameter')
         <class 'introspection.parameter.Parameter'>
-    
+
     Note that this function will not import any modules; the relevant module
     must already be present in :any:`sys.modules`.
 
@@ -495,12 +476,12 @@ def resolve_identifier(identifier: str) -> object:
 
     :param identifier: The identifier to resolve
     :returns: The object referenced by the identifier
-    :raises NameError: If the identifier doesn't reference anything
+    :raises InvalidIdentifier: If the identifier doesn't reference anything
     """
-    names = identifier.split('.')
+    names = identifier.split(".")
 
     for i in range(len(names)):
-        module_name = '.'.join(names[:i])
+        module_name = ".".join(names[:i])
 
         if module_name not in sys.modules:
             continue
@@ -513,8 +494,8 @@ def resolve_identifier(identifier: str) -> object:
                 break
         else:
             return obj
-    
-    raise NameError(f"Failed to resolve identifier {identifier!r}")
+
+    raise InvalidIdentifier(identifier)
 
 
 def is_sub_qualname(sub_name: str, base_name: str) -> bool:
@@ -533,36 +514,44 @@ def is_sub_qualname(sub_name: str, base_name: str) -> bool:
 
     .. versionadded:: 1.5
     """
-    sub_name = sub_name.split('.')
-    base_name = base_name.split('.')
+    sub_parts = sub_name.split(".")
+    base_parts = base_name.split(".")
 
-    return sub_name[:len(base_name)] == base_name
+    return sub_parts[: len(base_parts)] == base_parts
 
 
 def compile_function(
-    code,
+    code: Union[str, Iterable[str]],
     *,
-    globals_=None,
-    allow_builtins=True,
-    file_name='<string>',
+    globals_: Optional[typing.Dict[str, object]] = None,
+    allow_builtins: bool = True,
+    file_name: str = "<string>",
     **kwargs,
-):
+) -> Callable:
     if globals_ is None:
         globals_ = {}
-    
+
     if allow_builtins:
-        globals_['__builtins__'] = builtins
-    
+        globals_ = {
+            "__builtins__": builtins,
+            **globals_,
+        }
+
     if not isinstance(code, str):
-        code = '\n'.join(code)
-    
-    code = compile(code, file_name, 'exec', **kwargs)
+        code = "\n".join(code)
 
-    variables = set(globals_)
-    exec(code, globals_)
-    [function_name] = globals_.keys() - variables
+    code_obj = compile(code, file_name, "exec", **kwargs)
 
-    return globals_[function_name]
+    predefined_globals = set(globals_)
+    exec(code_obj, globals_)
+
+    for name in globals_.keys() - predefined_globals:
+        obj = globals_[name]
+
+        if isinstance(obj, types.FunctionType):
+            return obj
+
+    raise ValueError("No function found in code")
 
 
 def camel_to_snake(camel: str) -> str:
@@ -587,17 +576,16 @@ def camel_to_snake(camel: str) -> str:
         if is_upper:
             char = char.lower()
 
-            if (
-                not last_char_was_upper or
-                (i > 0 and i+1 < len(camel) and not camel[i+1].isupper())
+            if not last_char_was_upper or (
+                i > 0 and i + 1 < len(camel) and not camel[i + 1].isupper()
             ):
-                chars.append('_')
-        
+                chars.append("_")
+
         chars.append(char)
-        
+
         last_char_was_upper = is_upper
 
-    return ''.join(chars)
+    return "".join(chars)
 
 
 def snake_to_camel(snake: str) -> str:
@@ -617,14 +605,14 @@ def snake_to_camel(snake: str) -> str:
     last_char_was_underscore = True
 
     for char in snake:
-        if char == '_':
+        if char == "_":
             last_char_was_underscore = True
             continue
-        
+
         if last_char_was_underscore:
             char = char.upper()
             last_char_was_underscore = False
-        
+
         chars.append(char)
-    
-    return ''.join(chars)
+
+    return "".join(chars)
